--[[

"Solar System" class which holds the information for everything that the player
interacts with in the game. This class is basically what is commonly called the 
game class.

]]

System = class("System")

local gamera = require 'src/util/gamera'

function System:initialize()

	self.camera = gamera.new(-1000,-1000,2000,2000)
	self.camera:setScale(SCALE)
	
	self.world = love.physics.newWorld(0, 0, true)
	self.world:setCallbacks(beginContact, endContact, preSolve, postSolve)

	self.player = Player(0, 0, self.world, self.camera)
	
	self.background = Background(self.camera)
	
	self.actors = {}
	table.insert(self.actors, Planet(0, 50, 20, 1000, self.world))
	
	
end

function System:update(dt)

	self.world:update(dt)
	
	self.player:update(dt)
	
	
	for i = 1, #self.actors do
		
		self.actors[i]:update(dt)
		
	end
	
	-- update the camera so the player is always exactly in the center of the screen
	self.camera:setScale(SCALE)
	self.camera:setWorld(-1000 + self.player.body:getX(), -1000 + self.player.body:getY(), 2000, 2000)
	self:camFollowPlayer()


end

function System:draw()

	local l,t,w,h = self.camera:getWorld()
	
	self.camera:setScale(1)
	self.camera:draw(function()
	self.background:draw()
	end)
	self.camera:setScale(SCALE)
	
	self.camera:draw(function()
	
	
	for i = 1, #self.actors do
		
		self.actors[i]:draw()
		
	end
	
	self.player:draw()
	
	end)

end

function System:camFollowPlayer()

	local playerAngle = self.player.body:getAngle()
	local cameraAngle = self.camera:getAngle()
	self.camera:setAngle(playerAngle)
	local x1, y1 = self.player.body:getPosition()
	local positionX, positionY = 0, 0
	if love.keyboard.isDown('lctrl') and love.mouse.isDown('r') == false then
		local x2, y2 = self.camera:toWorld(love.mouse:getPosition())
		print(x2, y2, love.mouse:getPosition())
		positionX = (x1 + x2)/2
		positionY = (y1 + y2)/2
		self.camera:setPosition(
		positionX, positionY
		)
		
	else
		positionX = (x1)
		positionY = (y1)
		self.camera:setPosition(
		positionX, positionY
		)
	end

	
		
		

end

function System:keyPressed(key, unicode)
	
	self.player:keyPressed(key, unicode)

end

function System:mousepressed( x, y, mb )
	
	self.player:mousepressed( x, y, mb )

end

--[[ 
a is the first fixture object in the collision.
b is the second fixture object in the collision.
coll is the contact object created.
]]

function beginContact(a, b, coll)
	a:getUserData().parent:beginContact(b, coll)
	b:getUserData().parent:beginContact(a, coll)
end

function endContact(a, b, coll)
    a:getUserData().parent:endContact(b, coll)
	b:getUserData().parent:endContact(a, coll)
end

function preSolve(a, b, coll)
    
end

function postSolve(a, b, coll)
    
end
